/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global Ext, Sonatype, Nexus*/
define('Nexus/clm/StagingInitialization', ['Nexus/clm',
  'Nexus/staging/repository/activity/EventDetail',
  'Nexus/staging/repository/SummaryView',
  'Nexus/staging/repository/RepositoriesMediator'], function() {
  Ext.onReady(function() {
    Nexus.clm.ruleTemplate = NX.create('Ext.XTemplate',
        '<div class="nx-clm-status {messageClass:htmlEncode}">{message:htmlEncode}</div>',
        '<div class="nx-staging-repository-ActivityView-messagePanel-head">',
        '<tpl if="violationCount &gt; 0">Violations were found in <b>{violationCount:htmlEncode}</b> components.</tpl>',
        '<tpl if="violationCount == 0">No violations found.</tpl>',
        '</div>',
        '<div class="nx-clm-alerts">',
        '<tpl if="criticalCount &gt; 0">',
        '<div class="nx-clm-alert_box">',
        '<div class="nx-clm-chiclet critical">{criticalCount:htmlEncode}</div>',
        '<div class="nx-clm-alert_critical_text">CRITICAL</div>',
        '</div>',
        '</tpl>',
        '<tpl if="severeCount &gt; 0">',
        '<div class="nx-clm-alert_box">',
        '<div class="nx-clm-chiclet severe">{severeCount:htmlEncode}</div>',
        '<div class="nx-clm-alert_severe_text">SEVERE</div>',
        '</div>',
        '</tpl>',
        '<tpl if="moderateCount &gt; 0">',
        '<div class="nx-clm-alert_box">',
        '<div class="nx-clm-chiclet moderate">{moderateCount:htmlEncode}</div>',
        '<div class="nx-clm-alert_moderate_text">MODERATE</div>',
        '</div>',
        '</tpl>',
        '<a href="{detailedReport:htmlEncode}" target="_blank" class="view-report-button">View Full Report</a>',
        '</div>',
        '<br style="clear:both;"/>',
        {
          compiled : true
        }
    );

    Nexus.clm.stagingRule = function(event, detail) {
      if (detail.getPropertyValue('typeId') === 'iq-policy' || detail.getPropertyValue('typeId') === 'clm') {
        var message, messageClass;

        var status = detail.getPropertyValue('x.appScan.status');
        if(status === undefined)
        {
          NX.log.error("Unable to determine status for IQ Server result:" + detail);
          return undefined;
        }

        switch (status) {
          case 'PASS':
            message = 'Validation was successful!';
            messageClass = 'nx-clm-status_success';
            break;
          case 'WARN':
            message = 'There are warnings.';
            messageClass = 'nx-clm-status_warn';
            break;
          case 'FAIL':
            message = 'Validation failed.';
            messageClass = 'nx-clm-status_fail';
            break;
          default :
            Nexus.clm.showMessage("Unrecognized IQ Server status: "+ details.getPropertyValue('x.appScan.status'));
        }
        var violationCount = detail.getPropertyValue('x.appScan.violationCount') || 0;
        var criticalCount = detail.getPropertyValue('x.appScan.threatLevelCount.CRITICAL') || 0;
        var severeCount = detail.getPropertyValue('x.appScan.threatLevelCount.SEVERE') || 0;
        var moderateCount = detail.getPropertyValue('x.appScan.threatLevelCount.MODERATE') || 0;

        return NX.create('Ext.XTemplate',
            '{[ this.render() ]}',
            {
              compiled : true,
              render : function() {
                return Nexus.clm.ruleTemplate.apply({
                  message: message,
                  messageClass: messageClass,
                  detailedReport : detail.getPropertyValue('x.appScan.url'),
                  violationCount : violationCount,
                  criticalCount : criticalCount,
                  severeCount : severeCount,
                  moderateCount : moderateCount
                });
              }
            });
      }

      return undefined;
    };

    /**
     * The key for data collected in Staging form.
     * @type {string}
     */
    var key = 'CLMApplicationID';

    /**
     * The label to show in Staging form.
     * @type {string}
     */
    var label = 'IQ Server Application';

    // Attach SettingsView to navigation panel if current user has appropriate permissions
    Sonatype.Events.addListener('nexusNavigationInit', function (nexusPanel) {
      var p = Sonatype.lib.Permissions;
      if (p.checkPermission('nexus:settings', p.READ)) {
        Ext.Ajax.request({
          url :  Nexus.clm.resourcePath,
          method : 'GET',
          scope : self,
          suppressStatus: 404,
          success : function(response) {
            var clmSettings = Ext.decode(response.responseText);
            if (!Ext.isEmpty(clmSettings.url)) {
              Nexus.clm.serverUrl = clmSettings.url;
              nexusPanel.clmMenu = nexusPanel.add({
                enabled: true,
                sectionId: 'st-enterprise-staging',
                title: Nexus.clm.TITLE_DASHBOARD,
                href: encodeURI(Nexus.clm.serverUrl)
              });
              nexusPanel.doLayout();
            }
          }
        });
      }
    });

    //Enhance StagingProfile UI model with IQ Server metadata. Works in conjunction with the staging.profile.submitted event.
    Nexus.staging.repository.RepositoriesMediator.addListener('staging.profile.loaded', function(items, properties, dataRoot) {
      var clmAppPolicyURL;
      NX.log.debug("Enhancing UI with IQ Server details.");
      clmAppPolicyURL = new Ext.form.DisplayField({
        anchor: Sonatype.view.FIELD_OFFSET,
        allowBlank: true,
        hidden: !Ext.isDefined(Nexus.clm.serverUrl),
        appId: properties[key],
        showClmAppUrl: function () {
          var appId = clmAppPolicyURL.appId,
              url = Nexus.clm.serverUrl,
              text = 'IQ Server Dashboard';

          if (!Ext.isEmpty(appId)) {
            while (url.endsWith('/')) {
              url = url.substring(0, url.length - 1);
            }
            url += '/ui/links/application/' + encodeURIComponent(appId) + '/management';
            text = 'IQ Server Application Management';
          }
          clmAppPolicyURL.setValue('<a href="' + encodeURI(url) + '" target="_blank">' + text + '</a>');
        }
      });
      clmAppPolicyURL.showClmAppUrl();
      items.push([
        new Ext.form.ComboBox({
          fieldLabel: label,
          name: dataRoot + '.' + key,
          value: properties[key],
          width: 376,
          helpText: 'Select an IQ Server Application to enable, or (none) to disable policy enforcement for this profile.<br/>'
          + '<a href="http://links.sonatype.com/products/clm/nexus/home"  target="_blank">'
          + 'Make Nexus Staging even more powerful with Nexus IQ Server'
          + ' </a>',
          persistent: true,
          store: new Ext.data.JsonStore({
            id: 'id',
            fields: [
              { name: 'id'},
              { name: 'name', sortType: Ext.data.SortTypes.asUCString }
            ],
            sortInfo: {
              field: 'name',
              direction: 'ASC'
            },
            url: Nexus.clm.resourcePath + "/applications",
            autoLoad: true,
            listeners: {
              load: function (store) {
                store.insert(0, NX.create('Ext.data.Record', {id: '', name: '(none)'}));
              }
            }
          }),
          displayField: 'name',
          valueField: 'id',
          editable: false,
          forceSelection: true,
          mode: 'local',
          triggerAction: 'all',
          emptyText: 'Select...',
          selectOnFocus: true,
          allowBlank: true,
          listeners: {
            select: function (combo, record) {
              clmAppPolicyURL.appId = record.get('id');
              clmAppPolicyURL.showClmAppUrl();
            }
          }
        }),
        clmAppPolicyURL
      ]);
    });

    //Enhance StagingProfile UI model to serialize IQ Server metadata. Works in conjunction with the staging.profile.loaded event.
    Nexus.staging.repository.RepositoriesMediator.addListener('staging.profile.submitted', function(referenceData) {
      NX.log.debug("Enhancing StagingProfile submit data with IQ Server details.");
      referenceData[key] = null;
    });

    /**
     * @param value The encoded value with details of last IQ Server result.
     * @param showDetailedReport Whether or not a link to the full IQ Server results should be included
     * @return {*} view of the last IQ Server result.
     */
    function renderClmValues(value, showDetailedReport) {
      var values = value.split(',');
      NX.assert(values.length == 6, "Should have 6 comma separated values stored for IQ Server violations.");
      return Nexus.clm.chickletTemplate.apply({
        criticalCount : values[2],
        severeCount : values[3],
        moderateCount : values[4],
        detailedReport : showDetailedReport ? values[5] : null
      });
    }

    // enhance StagingRepository grid view with an additional column
    Nexus.staging.repository.RepositoriesMediator.addListener('staging.repositoriesGrid.configureColumns', function(columns) {
      NX.log.debug("Enhancing Repository grid view with IQ Server details.");
      columns.push({
        id : 'violations',
        width : 250,
        header : 'IQ Policy Violations',
        renderer : function(value, metaData, record) {
          var properties = record.data.properties;
          if (properties !== undefined) {
            var clmProperty;
            Ext.each(properties, function(property) {
              if (property.name == "IQ Policy Violations") {
                clmProperty = property.value;
                return false;
              }
            });
            if(clmProperty !== undefined)
            {
              return renderClmValues(clmProperty, false);
            }
          }
          return 'No IQ Server data recorded';
        }
      });
    });

    //Register renderer for IQ Server summary in StagingRepository summary view.
    Nexus.staging.repository.RepositoriesMediator.addListener('staging.repositorySummary.configureProperties', function(properties, values){
      Ext.each(values.properties, function(property){
        if(property.name && property.value && property.name === 'IQ Policy Violations')
        {
          NX.log.debug("Enhancing Repository summary view with IQ Server details.");
          for(var i = 0; i <  properties.length; i++)
          {
            if(properties[i].name === 'Activity')
            {
              break;
            }
          }
          properties.splice(i+1, 0, { name: 'IQ Policy Violations', value: renderClmValues(property.value, true) });
          return false;
        }
      });
    });

    //Register renderers for IQ Server rules in the Staging UI
    Nexus.staging.repository.activity.EventDetail.registerBodyTemplateFactory('ruleFailed', Nexus.clm.stagingRule);
    Nexus.staging.repository.activity.EventDetail.registerBodyTemplateFactory('rulePassed', Nexus.clm.stagingRule);
  });
});
